define({
    'name' : 'List of assumptions',
    'sub-menu' : [
        {
            'name' : 'Bounded',
            'snippet' : ['Q.bounded(x)',],
        },
        {
            'name' : 'Commutative',
            'snippet' : ['Q.commutative(x)',],
        },
        {
            'name' : 'Complex',
            'snippet' : ['Q.complex(x)',],
        },
        {
            'name' : 'Imaginary',
            'snippet' : ['Q.imaginary(x)',],
        },
        {
            'name' : 'Real',
            'snippet' : ['Q.real(x)',],
        },
        {
            'name' : 'Extended real',
            'snippet' : ['Q.extended_real(x)',],
        },
        {
            'name' : 'Integer',
            'snippet' : ['Q.integer(x)',],
        },
        {
            'name' : 'Odd',
            'snippet' : ['Q.odd(x)',],
        },
        {
            'name' : 'Even',
            'snippet' : ['Q.even(x)',],
        },
        {
            'name' : 'Prime',
            'snippet' : ['Q.prime(x)',],
        },
        {
            'name' : 'Composite',
            'snippet' : ['Q.composite(x)',],
        },
        {
            'name' : 'Zero',
            'snippet' : ['Q.zero(x)',],
        },
        {
            'name' : 'Nonzero',
            'snippet' : ['Q.nonzero(x)',],
        },
        {
            'name' : 'Rational',
            'snippet' : ['Q.rational(x)',],
        },
        {
            'name' : 'Algebraic',
            'snippet' : ['Q.algebraic(x)',],
        },
        {
            'name' : 'Transcendental',
            'snippet' : ['Q.transcendental(x)',],
        },
        {
            'name' : 'Irrational',
            'snippet' : ['Q.irrational(x)',],
        },
        {
            'name' : 'Finite',
            'snippet' : ['Q.finite(x)',],
        },
        {
            'name' : 'Infinite',
            'snippet' : ['Q.infinite(x)',],
        },
        {
            'name' : 'Infinitesimal',
            'snippet' : ['Q.infinitesimal(x)',],
        },
        {
            'name' : 'Negative',
            'snippet' : ['Q.negative(x)',],
        },
        {
            'name' : 'Nonnegative',
            'snippet' : ['Q.nonnegative(x)',],
        },
        {
            'name' : 'Positive',
            'snippet' : ['Q.positive(x)',],
        },
        {
            'name' : 'Nonpositive',
            'snippet' : ['Q.nonpositive(x)',],
        },
        {
            'name' : 'Hermitian',
            'snippet' : ['Q.hermitian(x)',],
        },
        {
            'name' : 'Antihermitian',
            'snippet' : ['Q.antihermitian(x)',],
        },
    ],
});
